import numpy
import struct

def get_string(l):
	return ''.join(chr(i) for i in l[0:l.index(0)])
  
def read_cache(filename):
	with open(filename, 'rb') as binfile:
		b = binfile.read()
		l = len(b)
		if l == 0 or l & 4095:
			raise Exception('size check cache file')

		h = struct.unpack("IIII", b[-16:])

		x = h[0] & 3;
		if x == 1:
			print('NIO_VECTOR', end='')
			s = h[1]
			dm = h[1:2]
		elif x == 2:
			print('NIO_MATRIX', end='')
			s = h[1] * h[2]
			dm = h[1:3]
		elif x == 3:
			print('NIO_CUBE', end='')
			s = h[1] * h[2] * h[3]
			dm = h[1:4]
		else:
			raise Exception('unsupported shape')

		if h[0] & 256:
			x = (h[0] & 255) >> 2;
			print('<char[{}]>'.format(x))
			ps = s
			s = s * x
			dtype = numpy.uint8
		else:
			x = (h[0] >> 2) & 15
			if x == 1:
				print('<char>', end='')
				dtype = numpy.uint8
			elif x == 2:
				print('<int>', end='')
				s = s * 4;
				dtype = numpy.int32
			elif x == 3:
				print('<long>', end='')
				s = s * 8;
				dtype = numpy.int64
			elif x == 4:
				print('<float>', end='')
				s = s * 4;
				dtype = numpy.float32
			elif x == 5:
				print('<double>', end='')
				s = s * 8;
				dtype = numpy.float64
			elif x == 6:
				print('<unsigned short>', end='')
				s = s * 2;
				dtype = numpy.uint16
			elif x == 7:
				print('<unsigned int>', end='')
				s = s * 4;
				dtype = numpy.uint32
			elif x == 8:
				print('<short>', end='')
				s = s * 2;
				dtype = numpy.int16
			else:
				raise Exception('unsupported data type')

		if l < ((s + 16 + 4095) & ~4095):
			raise Exception('header check cache file')

		dat = numpy.frombuffer(b[:s], dtype=dtype)
		print(dm)

		if h[0] & 256:
			dat = dat.reshape([ps, x])
			dat = numpy.array([get_string(list(dat[i])) for i in range(ps)])

		if len(dm) > 1:
			dat = dat.reshape(dm)

		return dat

def write_cache(array, f):
	binfile = open(f, 'wb')
	h = list(array.shape)
	l = len(h)
	if array.dtype == numpy.int32:
		h.insert(0, l + (2<<2))
	elif array.dtype == numpy.int64:
		h.insert(0, l + (3<<2))
	elif array.dtype == numpy.float32:
		h.insert(0, l + (4<<2))
	elif array.dtype == numpy.float64:
		h.insert(0, l + (5<<2))
	elif array.dtype == numpy.uint16:
		h.insert(0, l + (6<<2))
	elif array.dtype == numpy.uint32:
		h.insert(0, l + (7<<2))
	elif array.dtype == numpy.int16:
		h.insert(0, l + (8<<2))
	else:
		#assume char[N], hardcode N below
		ll = 32
		h.insert(0, 256 + l + (ll<<2))
		if l == 2:
			array.reshape([h[1] * h[2]])
		array = [(list(map(ord, list(i))) + [0]*32)[:32] for i in array.ravel()] 
		array = numpy.array(array, dtype=numpy.uint8)

	if l == 1:
		h.append(0)
		h.append(0)
	elif l == 2:
		h.append(0)

	array.astype(array.dtype).tofile(binfile)
	binfile.seek(((binfile.tell() + 16 + 4095) & ~4095) - 16)
	h = numpy.array(h, dtype=numpy.int32)
	h.astype(h.dtype).tofile(binfile)
	binfile.close()
